! ----------------------------------------------------------------------
!        Dissertation            DI KOFLER Peter 9126050        E086 869

!        Funktionendatei: Problem 06d (2-dim. System 1. Ord.)
!                         Example 2, \cite{ew1}, p. 139.

!        y(x) = [(1/2x)^4*exp(4-2x);(1/2x)^4*exp(4-2*x)*(4-2x)] (EW 0, 0)
!        y'(x) = [0 1; 0 0]*y(x)/x + x*[0; y + ...]
!        y(0) = [0; 0]
!        0 - 1.0
! ----------------------------------------------------------------------
MODULE fun
   USE bessel
   IMPLICIT NONE

!        Parameter
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   LOGICAL, PARAMETER :: linear = .true. 
   PRIVATE
   PUBLIC :: dfcn, linear, lrk, neqn, initial, fcn, true, truelocal
!        INTEGER neqn ... Dimension des Systems als Konstante
!        SUBROUTINE initial(xstart,xend,yx0) ... Anfangsbedingungen
!        REAL FUNTION fcn ... f(x,y)
!        REAL FUNTION true ... y(x)
!        REAL FUNTION truelocal(x,xstart,yx0) ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte

   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   CALL setupneu()      ! Neumannfunktionen vorbereiten
   xstart = 0.0d0
   xend = 1.0d0
   yx0 = (/ 0.0d0, 0.0d0 /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y), mit Maple ueberprueft

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0d0) THEN
      fcn = (/ 0.0d0, 0.0d0 /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(2)/x, x*4.0D0*y(1) + (x/2.0D0)**3/2.0D0* &
               exp(4.0D0-2.0D0*x)*(16.0D0-18.0D0*x) /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x)
!        echte Loesung y(x), mit Maple eingesetzt.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(neqn)

   true = (/ (x/2.0D0)**4*exp(4.0D0-2.0D0*x), &
             (x/2.0D0)**4*exp(4.0D0-2.0D0*x)*(4.0D0-2.0D0*x) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten, Loesungsschar,
!        fuer P06x Bsp. wo I_0(alpha) als allg. Lsg herauskommt.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, v(neqn), a, k0, k1, i0, i1
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   a = 2.0D0                  ! 8, 128, 32, 2
   v = yx0 - true(xstart)
   if (xstart == 0.0D0) THEN
      c1 = v(1)/besselI0(a*xstart)
      c2 = 0.D0
      a = a*x
      truelocal = (/ c1*besselI0(a), c1*a*besselI1(a) /) + true(x)
   ELSE
      c2 = a*xstart
      i0 = besselI0(c2)
      i1 = besselI1(c2)
      k0 = besselK0(c2)
      k1 = besselK1(c2)
      c1 = (v(1)*c2*k1+v(2)*k0)/(i0*k1+i1*k0)/c2
      c2 = (v(1)-c1*i0)/k0
      a = a*x
      truelocal = (/ c1*besselI0(a) + c2*besselK0(a), &
                    (c1*besselI1(a) - c2*besselK1(a))*a /) + true(x)
   END IF
   RETURN
END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0d0, 1.0d0/x /)
      dfcn(2,:) = (/ 0.0d0, 0.0d0 /)
   RETURN
END FUNCTION dfcn

END MODULE fun
