! ----------------------------------------------------------------------
!        Dissertation            DI KOFLER Peter 9126050        E086 869

!        Funktionendatei: Problem 33b (2-dim. System 1. Ord.)
!                         Example 2, \cite{ew1}, p. 139.

!        y(x) = [exp(64*(ln(2)+1+ln(t)-2t));exp(...)*(64-128*x)] (EW 0,0)
!        y'(x) = [0 1; 0 0]*y(x)/x + x*[0; ...]
!        y(0) = [0; 0]
!        y(1) = [.29585E-8;-.18934E-6]
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE

!        Parameter
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   LOGICAL, PARAMETER :: linear = .true. 
   PRIVATE
   PUBLIC :: dfcn, linear, lrk, neqn, initial, fcn, true, truelocal
!        INTEGER neqn ... Dimension des Systems als Konstante
!        SUBROUTINE initial(xstart,xend,yx0) ... Anfangsbedingungen
!        REAL FUNTION fcn ... f(x,y)
!        REAL FUNTION true ... y(x)
!        REAL FUNTION truelocal(x,xstart,yx0) ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte

   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0d0
   xend = 1.0d0
   yx0 = (/ 0.0d0, 0.0d0 /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y), mit Maple ueberprueft

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0d0) THEN
      fcn = (/ 0.0d0, 0.0d0 /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(2)/x, &
            exp( (log(2.0D0)+1.0D0-2.0D0*x)*64.0D0 + log(x)*63.0D0 ) * &
                           ( 4096.0D0 - x*(16512.0D0 - x*16384.0D0) ) /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x)
!        echte Loesung y(x), mit Maple eingesetzt.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(neqn)

   IF (x == 0.0d0) THEN
     true = (/ 0.0D0, 0.0D0 /)
   ELSE
     true = (/ exp( (log(2.0D0*x)+1.0D0-2.0D0*x) * 64.0D0), &
       exp( (log(2.0D0*x)+1.0D0-2.0D0*x) * 64.0D0) * (64.0D0-128.0D0*x) /)
   END IF
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten, Loesungsschar,
!        fuer lineare, konst. Koeff, l1=l2=0.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = yx0 - true(xstart)    ! was bleibt fuer Lsg. des homogenen Problems
   c2 = t(2)
   IF (c2 == 0.0D0) THEN
      c1 = t(1)
   ELSE
      c1 = t(1) - c2*log(xstart)
   END IF

   truelocal = (/ c1 + c2*log(x), c2 /) + true(x)
   RETURN
END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0d0, 1.0d0/x /)
      dfcn(2,:) = (/ 0.0d0, 0.0d0 /)
   RETURN
END FUNCTION dfcn

END MODULE fun
